@@ -18,7 +18,30 @@ struct ipa_head_ext {
 	uint8_t data[0];
 } __attribute__ ((packed));
 
+struct osmo_ipa_msgb_cb {
+	uint8_t proto;
+	uint8_t proto_ext;
+} __attribute__ ((packed));
+
+
+/* We don't just cast to 'struct osmo_ipa_msgb_cb *', because that would
+ * break the strict aliasing rule. Casting to a reference to a union with
+ * a compatible struct member seems to be allowed, though, see:
+ *	N1570 Committee Draft — April 12, 2011 ISO/IEC 9899:201x,
+ *	Section 6.5, §7 */
+#define OSMO_IPA_MSGB_CB(__msg)	(&((( \
+					union { \
+						unsigned long cb; \
+						struct osmo_ipa_msgb_cb _cb; \
+					} \
+				*)&((__msg)->cb[0]))->_cb))
+
+#define osmo_ipa_msgb_cb_proto(__x)	OSMO_IPA_MSGB_CB(__x)->proto
+#define osmo_ipa_msgb_cb_proto_ext(__x)	OSMO_IPA_MSGB_CB(__x)->proto_ext
+
 struct msgb *osmo_ipa_msg_alloc(int headroom);
+struct msgb *osmo_ipa_ext_msg_alloc(size_t headroom);
+
 void osmo_ipa_msg_push_header(struct msgb *msg, uint8_t proto);
 
 int osmo_ipa_process_msg(struct msgb *msg);
@@ -27,7 +50,6 @@ struct osmo_fd;
 struct tlv_parsed;
 
 int osmo_ipa_rcvmsg_base(struct msgb *msg, struct osmo_fd *bfd, int server);
-int osmo_ipa_idtag_parse(struct tlv_parsed *dec, unsigned char *buf, int len);
 int osmo_ipa_parse_unitid(const char *str, struct ipaccess_unit *unit_data);
 
 int ipaccess_send_pong(int fd);
@@ -41,4 +63,8 @@ struct msgb *ipa_cli_id_ack(void);
 
 int osmo_ipa_parse_msg_id_resp(struct msgb *msg, struct ipaccess_unit *unit_data);
 
+int osmo_ipa_segmentation_cb(struct msgb *msg);
+
+void osmo_ipa_msg_push_headers(struct msgb *msg, enum ipaccess_proto p, enum ipaccess_proto_ext pe);
+
 #endif
