@@ -52,18 +52,7 @@
 
 #include <osmocom/netif/sctp.h>
 
-
-/*! \addtogroup stream Osmocom Stream Socket (server side)
- *  @{
- *
- *  This code is intended to abstract any use of stream-type sockets,
- *  such as TCP and SCTP.  It offers server side implementation,
- *  fully integrated with the libosmocore select loop abstraction.
- */
-
-/*! \file stream_srv.c
- *  \brief Osmocom stream socket helpers (server side)
- */
+/*! \file stream_srv.c */
 
 #define LOGSLNK(link, level, fmt, args...) \
 	LOGP(DLINP, level, "SRV(%s,%s) " fmt, \
@@ -93,7 +82,7 @@ struct osmo_stream_srv_link {
 	int			sk_domain;
 	int			sk_type;
 	uint16_t		proto;
-	int (*accept_cb)(struct osmo_stream_srv_link *srv, int fd);
+	osmo_stream_srv_link_accept_cb_t accept_cb;
 	void			*data;
 	int			flags;
 	struct osmo_sock_init2_multiaddr_pars ma_pars;
@@ -174,11 +163,14 @@ error_close_socket:
 	return ret;
 }
 
-/*! \brief Create an Osmocom Stream Server Link
- *  A Stream Server Link is the listen()+accept() "parent" to individual
- *  Stream Servers
+/*! \addtogroup stream_srv
+ *  @{
+ */
+
+/*! Create an Osmocom Stream Server Link.
+ *  A Stream Server Link is the listen()+accept() "parent" to individual connections from remote clients.
  *  \param[in] ctx talloc allocation context
- *  \returns Stream Server Link with default values (TCP)
+ *  \returns Stream Server Link with default values (AF_UNSPEC, SOCK_STREAM, IPPROTO_TCP)
  */
 struct osmo_stream_srv_link *osmo_stream_srv_link_create(void *ctx)
 {
@@ -198,8 +190,9 @@ struct osmo_stream_srv_link *osmo_stream_srv_link_create(void *ctx)
 	return link;
 }
 
-/*! \brief Set a name on the srv_link object (used during logging)
- *  \param[in] link server link whose name is to be set
+/*! Set a name on the srv_link object (used during logging).
+ *  \param[in] link server link whose name is to be set.  The name is copied into the osmo_stream_srv_link, so
+ *  the caller memory is not required to be valid beyond the call of this function.
  *  \param[in] name the name to be set on link
  */
 void osmo_stream_srv_link_set_name(struct osmo_stream_srv_link *link, const char *name)
@@ -207,7 +200,7 @@ void osmo_stream_srv_link_set_name(struct osmo_stream_srv_link *link, const char
 	osmo_talloc_replace_string(link, &link->name, name);
 }
 
-/*! \brief Retrieve name previously set on the srv_link object (see osmo_stream_srv_link_set_name())
+/*! Retrieve name previously set on the srv_link object (see osmo_stream_srv_link_set_name()).
  *  \param[in] link server link whose name is to be retrieved
  *  \returns The name to be set on link; NULL if never set
  */
@@ -216,7 +209,7 @@ const char *osmo_stream_srv_link_get_name(const struct osmo_stream_srv_link *lin
 	return link->name;
 }
 
-/*! \brief Set the NODELAY socket option to avoid Nagle-like behavior
+/*! Set the NODELAY socket option to avoid Nagle-like behavior.
  *  Setting this to nodelay=true will automatically set the NODELAY
  *  socket option on any socket established via this server link, before
  *  calling the accept_cb()
@@ -231,7 +224,8 @@ void osmo_stream_srv_link_set_nodelay(struct osmo_stream_srv_link *link, bool no
 		link->flags &= ~OSMO_STREAM_SRV_F_NODELAY;
 }
 
-/*! \brief Set the local address to which we bind
+/*! Set the local address to which we bind.
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] link Stream Server Link to modify
  *  \param[in] addr Local IP address
  */
@@ -241,8 +235,9 @@ void osmo_stream_srv_link_set_addr(struct osmo_stream_srv_link *link,
 	osmo_stream_srv_link_set_addrs(link, &addr, 1);
 }
 
-/*! \brief Set the local address set to which we bind.
+/*! Set the local address set to which we bind.
  *  Useful for protocols allowing bind on more than one address (such as SCTP)
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] link Stream Server Link to modify
  *  \param[in] addr Local IP address
  *  \return negative on error, 0 on success
@@ -266,7 +261,8 @@ int osmo_stream_srv_link_set_addrs(struct osmo_stream_srv_link *link, const char
 	return 0;
 }
 
-/*! \brief Set the local port number to which we bind
+/*! Set the local port number to which we bind.
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] link Stream Server Link to modify
  *  \param[in] port Local port number
  */
@@ -277,7 +273,8 @@ void osmo_stream_srv_link_set_port(struct osmo_stream_srv_link *link,
 	link->flags |= OSMO_STREAM_SRV_F_RECONF;
 }
 
-/*! \brief Set the protocol for the stream server link
+/*! Set the protocol for the stream server link.
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] link Stream Server Link to modify
  *  \param[in] proto Protocol (like IPPROTO_TCP (default), IPPROTO_SCTP, ...)
  */
@@ -290,7 +287,8 @@ osmo_stream_srv_link_set_proto(struct osmo_stream_srv_link *link,
 }
 
 
-/*! \brief Set the socket type for the stream server link
+/*! Set the socket type for the stream server link.
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] link Stream Server Link to modify
  *  \param[in] type Socket Type (like SOCK_STREAM (default), SOCK_SEQPACKET, ...)
  *  \returns zero on success, negative on error.
@@ -309,7 +307,8 @@ int osmo_stream_srv_link_set_type(struct osmo_stream_srv_link *link, int type)
 	return 0;
 }
 
-/*! \brief Set the socket type for the stream server link
+/*! Set the socket type for the stream server link.
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] link Stream Server Link to modify
  *  \param[in] type Socket Domain (like AF_UNSPEC (default for IP), AF_UNIX, AF_INET, ...)
  *  \returns zero on success, negative on error.
@@ -330,7 +329,7 @@ int osmo_stream_srv_link_set_domain(struct osmo_stream_srv_link *link, int domai
 	return 0;
 }
 
-/*! \brief Set application private data of the stream server link
+/*! Set application private data of the stream server link.
  *  \param[in] link Stream Server Link to modify
  *  \param[in] data User-specific data (available in call-back functions) */
 void
@@ -340,7 +339,7 @@ osmo_stream_srv_link_set_data(struct osmo_stream_srv_link *link,
 	link->data = data;
 }
 
-/*! \brief Get application private data of the stream server link
+/*! Retrieve application private data of the stream server link.
  *  \param[in] link Stream Server Link to modify
  *  \returns Application private data, as set by \ref osmo_stream_cli_set_data() */
 void *osmo_stream_srv_link_get_data(struct osmo_stream_srv_link *link)
@@ -402,7 +401,9 @@ static char *get_local_sockname_buf(char *buf, size_t buf_len, const struct osmo
 	}
 }
 
-/*! \brief Get description of the stream server link e. g. 127.0.0.1:1234
+/*! Retrieve description of the stream server link e. g. 127.0.0.1:1234.
+ *  Calling this function will build a string that describes the socket in terms of its local/remote
+ *  address/port.  The returned name is stored in a static buffer; it is hence not re-entrant or thread-safe.
  *  \param[in] link Stream Server Link to examine
  *  \returns Link description or NULL in case of error */
 char *osmo_stream_srv_link_get_sockname(const struct osmo_stream_srv_link *link)
@@ -414,7 +415,7 @@ char *osmo_stream_srv_link_get_sockname(const struct osmo_stream_srv_link *link)
 	return buf;
 }
 
-/*! \brief Get Osmocom File Descriptor of the stream server link
+/*! Retrieve Osmocom File Descriptor of the stream server link.
  *  \param[in] link Stream Server Link
  *  \returns Pointer to \ref osmo_fd */
 struct osmo_fd *
@@ -423,7 +424,7 @@ osmo_stream_srv_link_get_ofd(struct osmo_stream_srv_link *link)
 	return &link->ofd;
 }
 
-/*! \brief Get File Descriptor of the stream server link
+/*! Retrieve  File Descriptor of the stream server link.
  *  \param[in] conn Stream Server Link
  *  \returns file descriptor or negative on error */
 int osmo_stream_srv_link_get_fd(const struct osmo_stream_srv_link *link)
@@ -431,7 +432,10 @@ int osmo_stream_srv_link_get_fd(const struct osmo_stream_srv_link *link)
 	return link->ofd.fd;
 }
 
-/*! \brief Set the accept() call-back of the stream server link
+/*! Set the accept() call-back of the stream server link.
+ *  The provided call-back will be called whenever a new inbound connection
+ *  is accept()ed.  The call-back then typically creates a new osmo_stream_srv.
+ *  If the call-back returns a negative value, the file descriptor will be closed.
  *  \param[in] link Stream Server Link
  *  \param[in] accept_cb Call-back function executed upon accept() */
 void osmo_stream_srv_link_set_accept_cb(struct osmo_stream_srv_link *link,
@@ -441,7 +445,7 @@ void osmo_stream_srv_link_set_accept_cb(struct osmo_stream_srv_link *link,
 	link->accept_cb = accept_cb;
 }
 
-/*! \brief Destroy the stream server link. Closes + Releases Memory.
+/*! Destroy the stream server link. Closes + Releases Memory.
  *  \param[in] link Stream Server Link */
 void osmo_stream_srv_link_destroy(struct osmo_stream_srv_link *link)
 {
@@ -449,8 +453,8 @@ void osmo_stream_srv_link_destroy(struct osmo_stream_srv_link *link)
 	talloc_free(link);
 }
 
-/*! \brief Open the stream server link.  This actually initializes the
- *  underlying socket and binds it to the configured ip/port
+/*! Open the stream server link.  This actually initializes the
+ *  underlying socket and binds it to the configured ip/port.
  *  \param[in] link Stream Server Link to open
  *  \return negative on error, 0 on success */
 int osmo_stream_srv_link_open(struct osmo_stream_srv_link *link)
@@ -504,7 +508,7 @@ int osmo_stream_srv_link_open(struct osmo_stream_srv_link *link)
 	return 0;
 }
 
-/*! \brief Check whether the stream server link is opened
+/*! Check whether the stream server link is opened.
  *  \param[in] link Stream Server Link to check */
 bool osmo_stream_srv_link_is_opened(const struct osmo_stream_srv_link *link)
 {
@@ -517,7 +521,7 @@ bool osmo_stream_srv_link_is_opened(const struct osmo_stream_srv_link *link)
 	return true;
 }
 
-/*! \brief Close the stream server link and unregister from select loop
+/*! Close the stream server link and unregister from select loop.
  *  Does not destroy the server link, merely closes it!
  *  \param[in] link Stream Server Link to close */
 void osmo_stream_srv_link_close(struct osmo_stream_srv_link *link)
@@ -530,6 +534,12 @@ void osmo_stream_srv_link_close(struct osmo_stream_srv_link *link)
 	link->ofd.fd = -1;
 }
 
+/*! Set given parameter of stream_srv_link to given value.
+ *  \param[in] cli stream client on which to set parameter.
+ *  \param[in] par identifier of the parameter to be set.
+ *  \param[in] val value of the parameter to be set.
+ *  \param[in] val_len length of the parameter value.
+ *  \returns 0 in success; negative -errno on error. */
 int osmo_stream_srv_link_set_param(struct osmo_stream_srv_link *link, enum osmo_stream_srv_link_param par,
 				   void *val, size_t val_len)
 {
@@ -573,6 +583,8 @@ int osmo_stream_srv_link_set_param(struct osmo_stream_srv_link *link, enum osmo_
 	return 0;
 }
 
+/*! @} */
+
 #define OSMO_STREAM_SRV_F_FLUSH_DESTROY	(1 << 0)
 
 struct osmo_stream_srv {
@@ -585,36 +597,45 @@ struct osmo_stream_srv {
 		struct osmo_io_fd		*iofd;
 	};
 	struct llist_head		tx_queue;
-	int (*closed_cb)(struct osmo_stream_srv *peer);
-	int (*read_cb)(struct osmo_stream_srv *peer);
-	int (*iofd_read_cb)(struct osmo_stream_srv *peer, struct msgb *msg);
+	osmo_stream_srv_closed_cb_t	closed_cb;
+	osmo_stream_srv_read_cb_t	read_cb;
+	osmo_stream_srv_read_cb2_t	iofd_read_cb;
 	void				*data;
 	int				flags;
 };
 
+/*! \addtogroup stream_srv
+ *  @{
+ */
+
 static void stream_srv_iofd_read_cb(struct osmo_io_fd *iofd, int res, struct msgb *msg)
 {
 	struct osmo_stream_srv *conn = osmo_iofd_get_data(iofd);
-	LOGSSRV(conn, LOGL_DEBUG, "message received (res=%d)\n", res);
 
-	if (OSMO_UNLIKELY(res <= 0)) {
-		/* This connection is dead, destroy it. */
-		msgb_free(msg);
-		osmo_stream_srv_destroy(conn);
-	} else {
-		if (conn->flags & OSMO_STREAM_SRV_F_FLUSH_DESTROY) {
-			LOGSSRV(conn, LOGL_INFO, "Connection is being flushed and closed; ignoring received message\n");
-			msgb_free(msg);
-			if (osmo_iofd_txqueue_len(iofd) == 0)
-				osmo_stream_srv_destroy(conn);
-			return;
-		}
-
-		if (conn->iofd_read_cb)
-			conn->iofd_read_cb(conn, msg);
-		else
-			msgb_free(msg);
+	switch (res) {
+	case -EPIPE:
+	case -ECONNRESET:
+		LOGSSRV(conn, LOGL_ERROR, "lost connection with client (%d)\n", res);
+		break;
+	case 0:
+		LOGSSRV(conn, LOGL_NOTICE, "connection closed with client\n");
+		break;
+	default:
+		LOGSSRV(conn, LOGL_DEBUG, "received %d bytes from client\n", res);
+		break;
 	}
+	if (OSMO_UNLIKELY(conn->flags & OSMO_STREAM_SRV_F_FLUSH_DESTROY)) {
+		LOGSSRV(conn, LOGL_INFO, "Connection is being flushed and closed; ignoring received message\n");
+		msgb_free(msg);
+		if (osmo_iofd_txqueue_len(iofd) == 0)
+			osmo_stream_srv_destroy(conn);
+		return;
+	}
+
+	if (conn->iofd_read_cb)
+		conn->iofd_read_cb(conn, res, msg);
+	else
+		msgb_free(msg);
 }
 
 static void stream_srv_iofd_write_cb(struct osmo_io_fd *iofd, int res, struct msgb *msg)
@@ -642,27 +663,32 @@ static void stream_srv_iofd_recvmsg_cb(struct osmo_io_fd *iofd, int res, struct
 	LOGSSRV(conn, LOGL_DEBUG, "message received (res=%d)\n", res);
 
 	res = stream_iofd_sctp_recvmsg_trailer(iofd, msg, res, msgh);
-	if (res == -EAGAIN)
-		return;
 
-	if (OSMO_UNLIKELY(res <= 0)) {
-		/* This connection is dead, destroy it. */
-		msgb_free(msg);
-		osmo_stream_srv_destroy(conn);
-	} else {
-		if (conn->flags & OSMO_STREAM_SRV_F_FLUSH_DESTROY) {
-			LOGSSRV(conn, LOGL_INFO, "Connection is being flushed and closed; ignoring received message\n");
-			msgb_free(msg);
-			if (osmo_iofd_txqueue_len(iofd) == 0)
-				osmo_stream_srv_destroy(conn);
-			return;
-		}
-
-		if (conn->iofd_read_cb)
-			conn->iofd_read_cb(conn, msg);
-		else
-			msgb_free(msg);
+	switch (res) {
+	case -EPIPE:
+	case -ECONNRESET:
+		LOGSSRV(conn, LOGL_ERROR, "lost connection with client (%d)\n", res);
+		break;
+	case 0:
+		LOGSSRV(conn, LOGL_NOTICE, "connection closed with client\n");
+		break;
+	default:
+		if (OSMO_LIKELY(res > 0))
+			LOGSSRV(conn, LOGL_DEBUG, "received %u bytes from client\n", res);
+		break;
 	}
+	if (OSMO_UNLIKELY(conn->flags & OSMO_STREAM_SRV_F_FLUSH_DESTROY)) {
+		LOGSSRV(conn, LOGL_INFO, "Connection is being flushed and closed; ignoring received message\n");
+		msgb_free(msg);
+		if (osmo_iofd_txqueue_len(iofd) == 0)
+			osmo_stream_srv_destroy(conn);
+		return;
+	}
+
+	if (conn->iofd_read_cb)
+		conn->iofd_read_cb(conn, res, msg);
+	else
+		msgb_free(msg);
 }
 
 static const struct osmo_io_ops srv_ioops_sctp = {
@@ -777,7 +803,16 @@ static int osmo_stream_srv_cb(struct osmo_fd *ofd, unsigned int what)
 	return rc;
 }
 
-/*! \brief Create a Stream Server inside the specified link
+
+/*! Create a legacy osmo_fd mode Stream Server inside the specified link.
+ *
+ *  This is the function an application traditionally calls from within the
+ *  accept_cb call-back of the osmo_stream_srv_link.  It creates a new
+ *  osmo_stream_srv within that link.
+ *
+ *  New users/programs should use osmo_stream_srv_create2 to operate in osmo_io
+ *  mode instead.
+ *
  *  \param[in] ctx talloc allocation context from which to allocate
  *  \param[in] link Stream Server Link to which we belong
  *  \param[in] fd system file descriptor of the new connection
@@ -786,10 +821,10 @@ static int osmo_stream_srv_cb(struct osmo_fd *ofd, unsigned int what)
  *  \param[in] data User data to save in the new Stream Server struct
  *  \returns Stream Server in case of success; NULL on error */
 struct osmo_stream_srv *
-osmo_stream_srv_create(void *ctx, struct osmo_stream_srv_link *link,
-	int fd,
-	int (*read_cb)(struct osmo_stream_srv *conn),
-	int (*closed_cb)(struct osmo_stream_srv *conn), void *data)
+osmo_stream_srv_create(void *ctx, struct osmo_stream_srv_link *link, int fd,
+		       osmo_stream_srv_read_cb_t read_cb,
+		       osmo_stream_srv_closed_cb_t closed_cb,
+		       void *data)
 {
 	struct osmo_stream_srv *conn;
 
@@ -817,7 +852,12 @@ osmo_stream_srv_create(void *ctx, struct osmo_stream_srv_link *link,
 	return conn;
 }
 
-/*! \brief Create a Stream Server inside the specified link
+/*! Create an osmo_iofd mode Stream Server inside the specified link.
+ *
+ *  This is the function an application typically calls from within the
+ *  accept_cb call-back of the osmo_stream_srv_link.  It creates a new
+ *  osmo_stream_srv in osmo_io mode within that link.
+ *
  *  \param[in] ctx talloc allocation context from which to allocate
  *  \param[in] link Stream Server Link to which we belong
  *  \param[in] fd system file descriptor of the new connection
@@ -863,8 +903,9 @@ osmo_stream_srv_create2(void *ctx, struct osmo_stream_srv_link *link, int fd, vo
 	return conn;
 }
 
-/*! \brief Set a name on the srv object (used during logging)
- *  \param[in] conn server whose name is to be set
+/*! Set a name on the srv object (used during logging).
+ *  \param[in] conn server whose name is to be set. The name is copied into the osmo_stream_srv_link, so
+ *  the caller memory is not required to be valid beyond the call of this function.
  *  \param[in] name the name to be set on conn
  */
 void osmo_stream_srv_set_name(struct osmo_stream_srv *conn, const char *name)
@@ -874,7 +915,7 @@ void osmo_stream_srv_set_name(struct osmo_stream_srv *conn, const char *name)
 		osmo_iofd_set_name(conn->iofd, name);
 }
 
-/*! \brief Retrieve name previously set on the srv object (see osmo_stream_srv_set_name())
+/*! Retrieve name previously set on the srv object (see osmo_stream_srv_set_name()).
  *  \param[in] conn server whose name is to be retrieved
  *  \returns The name to be set on conn; NULL if never set
  */
@@ -883,26 +924,36 @@ const char *osmo_stream_srv_get_name(const struct osmo_stream_srv *conn)
 	return conn->name;
 }
 
-/*! \brief Set the call-back function when data was read from the stream server socket
- *  Only for osmo_stream_srv created with osmo_stream_srv_create2()
+/*! Set the call-back function for incoming data on an osmo_io stream_srv.
+ *
+ *  This function only works with osmo_stream_srv in osmo_io mode, created by osmo_stream_srv_create2()!
+ *
+ *  Whenever data is received on the osmo_stram_srv, the read_cb call-back function of the user application is
+ *  called.
+ *
  *  \param[in] conn Stream Server to modify
  *  \param[in] read_cb Call-back function to be called when data was read */
-void osmo_stream_srv_set_read_cb(struct osmo_stream_srv *conn, int (*read_cb)(struct osmo_stream_srv *conn, struct msgb *msg))
+void osmo_stream_srv_set_read_cb(struct osmo_stream_srv *conn,
+				 osmo_stream_srv_read_cb2_t read_cb)
 {
 	OSMO_ASSERT(conn && conn->mode == OSMO_STREAM_MODE_OSMO_IO);
 	conn->iofd_read_cb = read_cb;
 }
 
-/*! \brief Set the call-back function called when the stream server socket was closed
+/*! Set the call-back function called when the stream server socket was closed.
+ *  Whenever the socket was closed (network error, client disconnect, etc.), the user-provided
+ *  call-back function given here is called.  This is typically used by the application to clean up any of its
+ *  internal state related to this specific client/connection.
  *  \param[in] conn Stream Server to modify
  *  \param[in] closed_cb Call-back function to be called when the connection was closed */
-void osmo_stream_srv_set_closed_cb(struct osmo_stream_srv *conn, int (*closed_cb)(struct osmo_stream_srv *conn))
+void osmo_stream_srv_set_closed_cb(struct osmo_stream_srv *conn,
+				   osmo_stream_srv_closed_cb_t closed_cb)
 {
 	OSMO_ASSERT(conn);
 	conn->closed_cb = closed_cb;
 }
 
-/*! \brief Prepare to send out all pending messages on the connection's Tx queue
+/*! Prepare to send out all pending messages on the connection's Tx queue.
  *  and then automatically destroy the stream with osmo_stream_srv_destroy().
  *  This function disables queuing of new messages on the connection and also
  *  disables reception of new messages on the connection.
@@ -912,7 +963,7 @@ void osmo_stream_srv_set_flush_and_destroy(struct osmo_stream_srv *conn)
 	conn->flags |= OSMO_STREAM_SRV_F_FLUSH_DESTROY;
 }
 
-/*! \brief Set application private data of the stream server
+/*! Set application private data of the stream server.
  *  \param[in] conn Stream Server to modify
  *  \param[in] data User-specific data (available in call-back functions) */
 void
@@ -922,12 +973,20 @@ osmo_stream_srv_set_data(struct osmo_stream_srv *conn,
 	conn->data = data;
 }
 
-/*! \brief Set the segmentation callback for target osmo_stream_srv structure.
- * The connection has to have been established prior to calling this function.
+/*! Set the segmentation callback for target osmo_stream_srv structure.
+ *
+ *  A segmentation call-back can optionally be used when a packet based protocol (like TCP) is used within a
+ *  STREAM style socket that does not preserve message boundaries within the stream.  If a segmentation
+ *  call-back is given, the osmo_stream_srv library code will makes sure that the read_cb called only for
+ *  complete single messages, and not arbitrary segments of the stream.
+ *
+ *  This function only works with osmo_stream_srv in osmo_io mode, created by osmo_stream_srv_create2()!
+  * The connection has to have been established prior to calling this function.
+  *
  *  \param[in,out] conn Target Stream Server to modify
  *  \param[in] segmentation_cb Segmentation callback to be set */
 void osmo_stream_srv_set_segmentation_cb(struct osmo_stream_srv *conn,
-					int (*segmentation_cb)(struct msgb *msg))
+					 osmo_stream_srv_segmentation_cb_t segmentation_cb)
 {
 	/* Note that the following implies that iofd != NULL, since
 	 * osmo_stream_srv_create2() creates the iofd member, too */
@@ -940,7 +999,7 @@ void osmo_stream_srv_set_segmentation_cb(struct osmo_stream_srv *conn,
 	osmo_iofd_set_ioops(conn->iofd, &conn_ops);
 }
 
-/*! \brief Get application private data of the stream server
+/*! Retrieve application private data of the stream server
  *  \param[in] conn Stream Server
  *  \returns Application private data, as set by \ref osmo_stream_srv_set_data() */
 void *osmo_stream_srv_get_data(struct osmo_stream_srv *conn)
@@ -948,7 +1007,8 @@ void *osmo_stream_srv_get_data(struct osmo_stream_srv *conn)
 	return conn->data;
 }
 
-/*! \brief Get the stream server socket description.
+/*! Retrieve the stream server socket description.
+ *  The returned name is stored in a static buffer; it is hence not re-entrant or thread-safe!
  *  \param[in] cli Stream Server to examine
  *  \returns Socket description or NULL in case of error */
 const char *osmo_stream_srv_get_sockname(const struct osmo_stream_srv *conn)
@@ -961,7 +1021,7 @@ const char *osmo_stream_srv_get_sockname(const struct osmo_stream_srv *conn)
 	return buf;
 }
 
-/*! \brief Get Osmocom File Descriptor of the stream server
+/*! Retrieve Osmocom File Descriptor of a stream server in osmo_fd mode.
  *  \param[in] conn Stream Server
  *  \returns Pointer to \ref osmo_fd */
 struct osmo_fd *
@@ -971,7 +1031,7 @@ osmo_stream_srv_get_ofd(struct osmo_stream_srv *conn)
 	return &conn->ofd;
 }
 
-/*! \brief Get File Descriptor of the stream server
+/*! Retrieve File Descriptor of the stream server
  *  \param[in] conn Stream Server
  *  \returns file descriptor or negative on error */
 int
@@ -989,7 +1049,18 @@ osmo_stream_srv_get_fd(const struct osmo_stream_srv *conn)
 	return -EINVAL;
 }
 
-/*! \brief Get the master (Link) from a Stream Server
+/*! Retrieve osmo_io descriptor of the stream server socket.
+ *  This function must not be called on a stream server in legacy osmo_fd mode!
+ *  \param[in] srv Stream Server of which we want to obtain the osmo_io descriptor
+ *  \returns osmo_io_fd of stream server. */
+struct osmo_io_fd *
+osmo_stream_srv_get_iofd(const struct osmo_stream_srv *srv)
+{
+	OSMO_ASSERT(srv->mode == OSMO_STREAM_MODE_OSMO_IO);
+	return srv->iofd;
+}
+
+/*! Retrieve the master (Link) from a Stream Server.
  *  \param[in] conn Stream Server of which we want to know the Link
  *  \returns Link through which the given Stream Server is established */
 struct osmo_stream_srv_link *osmo_stream_srv_get_master(struct osmo_stream_srv *conn)
@@ -997,11 +1068,10 @@ struct osmo_stream_srv_link *osmo_stream_srv_get_master(struct osmo_stream_srv *
 	return conn->srv;
 }
 
-/*! \brief Destroy given Stream Server
- *  This function closes the Stream Server socket, unregisters from
- *  select loop, invokes the connection's closed_cb() callback to allow API
- *  users to clean up any associated state they have for this connection,
- *  and then de-allocates associated memory.
+/*! Destroy given Stream Server.
+ *  This function closes the Stream Server socket, unregisters from the underlying I/O mechanism, invokes the
+ *  connection's closed_cb() callback to allow API users to clean up any associated state they have for this
+ *  connection, and then de-allocates associated memory.
  *  \param[in] conn Stream Server to be destroyed */
 void osmo_stream_srv_destroy(struct osmo_stream_srv *conn)
 {
@@ -1024,7 +1094,7 @@ void osmo_stream_srv_destroy(struct osmo_stream_srv *conn)
 	talloc_free(conn);
 }
 
-/*! \brief Enqueue data to be sent via an Osmocom stream server
+/*! Enqueue data to be sent via an Osmocom stream server.
  *  \param[in] conn Stream Server through which we want to send
  *  \param[in] msg Message buffer to enqueue in transmit queue */
 void osmo_stream_srv_send(struct osmo_stream_srv *conn, struct msgb *msg)
@@ -1057,11 +1127,15 @@ void osmo_stream_srv_send(struct osmo_stream_srv *conn, struct msgb *msg)
 	}
 }
 
-/*! \brief Receive data via Osmocom stream server
+/*! Receive data via an Osmocom stream server in osmo_fd mode.
  *  \param[in] conn Stream Server from which to receive
  *  \param msg pre-allocate message buffer to which received data is appended
  *  \returns number of bytes read, negative on error.
  *
+ *  Application programs using the legacy osmo_fd mode of osmo_stream_srv will use
+ *  this function to read/receive from a stream socket after they have been notified that
+ *  it is readable (via select/poll).
+ *
  *  If conn is an SCTP connection, additional specific considerations shall be taken:
  *  - msg->cb is always filled with SCTP ppid, and SCTP stream values, see msgb_sctp_*() APIs.
  *  - If an SCTP notification was received when reading from the SCTP socket,
@@ -1077,6 +1151,7 @@ int osmo_stream_srv_recv(struct osmo_stream_srv *conn, struct msgb *msg)
 	int ret;
 	OSMO_ASSERT(conn);
 	OSMO_ASSERT(msg);
+	OSMO_ASSERT(conn->mode == OSMO_STREAM_MODE_OSMO_FD);
 
 	switch (conn->srv->sk_domain) {
 	case AF_UNIX:
