@@ -906,7 +906,7 @@ int main(void)
 	osmo_gettimeofday_override_time.tv_usec = 0;
 
 	osmo_init_logging2(tall_test, &osmo_stream_test_log_info);
-	log_set_log_level(osmo_stderr_target, LOGL_INFO);
+	log_set_log_level(osmo_stderr_target, LOGL_DEBUG);
 	log_set_use_color(osmo_stderr_target, 0);
 	log_set_print_category_hex(osmo_stderr_target, 0);
 	log_set_print_category(osmo_stderr_target, 0);
